
/******************************************************
 *                                                    *
 * FPAC project.            FPAC WP EDITION           *
 *                                                    *
 * Parts of code from different sources of ax25-utils *
 *                                                    *
 * F6FBB 05-1997                                      *
 *                                                    *
 ******************************************************/

/******************************************************
 * 12/05/97 1.00 F6FBB First draft !
 * 07/09/97 1.01 F1OAT fpacwpd mode
 ******************************************************/
 
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <signal.h>
#include <syslog.h>
#include <ctype.h>

#include <sys/time.h>
#include <sys/types.h>
#include <sys/fcntl.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/socket.h>

#include <linux/ax25.h>
#include <linux/rose.h>

#include "axutils.h"
#include "axconfig.h"

#include "wp.h"

int cr = 0;

/*** Prototypes *******************/
static char *my_date(time_t date);
static void readline(char *, int);

#define CR() printf( (cr) ? "\r" : "\n"); 

int main(int ac, char **av)
{
	int p;
	int end = 0;
	int error = 0;
	char line[80];
	char command[80];
	char str[80];
	char *call;
	ax25_address addr;
	wp_t wp;

	if (wp_open("WPEDIT")) {
		perror("Cannot open WP service");
		exit(1);
	}
	
	while ((p = getopt(ac, av, "cl:")) != -1) 
	{
		switch (p)
		{
		case 'c' :
			cr = 1;
			break;
		case '?' :
			fprintf(stderr, "usage: wpedit [-c] callsign\n");
			return(1);
		}
	}

	if (optind == ac)
	{
		printf("Callsign is missing");
		CR();
		return(1);
	}

	call = strupr(av[optind]);

	if (convert_call_entry(call, addr.ax25_call) != 0)
	{
		printf("Invalid callsign %s", call); CR();
		return(1);
	}

	memset(&wp, 0, sizeof(wp_t));

	if (wp_get(&addr, &wp))
	{
		int rep;

		for (;;) 
		{
			CR();
			printf("Callsign %s not in database. Create [Y-N] ? ", call);
			fflush(stdout);
			readline(line, sizeof(line));

			rep = toupper(line[0]);

			if ((rep == 'Y') || (rep == 'N'))
				break;
		}

		if (rep == 'N')
			return(0);

		wp.address.srose_call = addr;

	}

	CR();

	while (!end)
	{
		int i;
		int len;
		char *add;
		char dnic[5];

		printf("Callsign  Last update       DNIC address digis"); CR();

		add = rose2asc(&wp.address.srose_addr);
		strncpy(dnic, add, 4); dnic[4] = '\0';

		printf("%-9s %s => %s %s ", 
		call, my_date(wp.date), dnic, add+4);

		for (i = 0 ; i < wp.address.srose_ndigis ; i++)
		{
			printf("%s ", ax2asc(&wp.address.srose_digis[i]));
		}

		printf("%s %s", wp.name, wp.city);
		CR(); CR();

		printf("WP-Editing %s : (A)ddress (D)igi (W)rite (B)ye", call); CR();
		fflush(stdout);

		readline(line, sizeof(line));

		command[0] = str[0] = '\0';

		sscanf(line, "%s %s", command, str);

		switch(toupper(command[0]))
		{
		case 'A':
			len = strlen(str);

			if (len == 0)
				error = 1;
			else if (len != 10)
				error = 2;
			else if (strcspn(str, "0123456789"))
				error = 3;
			else
			{
				convert_rose_address(str, wp.address.srose_addr.rose_addr);
			}
			break;
		case 'B':
			end = 1;
			break;
		case 'D':
			if (*str)
			{
				char *p = strtok(str, " ,");
				while (p)
				{
					if (convert_call_entry(p, wp.address.srose_digis[0].ax25_call) != 0)
					{
						printf("Invalid callsign %s", str); 
						CR();
						wp.address.srose_ndigis = 0;
						break;
					}
					++wp.address.srose_ndigis;
					if (wp.address.srose_ndigis == ROSE_MAX_DIGIS)
					{
						printf("Too many digis (max 5)"); 
						CR();
						wp.address.srose_ndigis = 0;
						break;
					}
					p = strtok(NULL, " ,");
				}
			}
			else
			{
				wp.address.srose_ndigis = 0;
			}
			break;
		case 'W':
			wp_set(&wp);
			printf("%s WP record updated", call); CR();
			wp_get(&wp.address.srose_call, &wp);
			break;
		default :
			printf("Unkonwn command %s", line); CR();
			break;
		}

		if (error)
		{
			printf("*** Error : ");
			switch(error)
			{
			case 1:
				printf("Address missing");
				break;
			case 2:
				printf("Address must have 10 digits");
				break;
			case 3:
				printf("Address is only digits");
				break;
			}

			error = 0;
			CR();
		}
	}

	return(0);
}

static char *my_date(time_t date)
{
	static char buf[20];
	struct tm *sdate;

	sdate = localtime (&date);
	sprintf(buf, "%02d/%02d/%02d %02d:%02d", 
		sdate->tm_mday,
		sdate->tm_mon + 1, 
		sdate->tm_year%100,
		sdate->tm_hour,
		sdate->tm_min);
	return(buf);
}

void readline(char *buf, int len)
{
	int nb;
	
	nb = read(STDIN_FILENO, buf, len-1);
	if (nb <= 0)
		exit(1);

	buf[nb] = '\0';
}
